#include-once
#include <String.au3>
#include <_XMLDomWrapper.au3>

;By Foxhound - GPL v3.

Global Const $_YT_TEMP_XML_DIR = @TempDir & "\gData.xml"

; #FUNCTION# =========================================================================================================
; Name...........: _YTGetFormatMap
; Description ...: Locates and isolates the fmt_url_map JS object from YouTube source.
; Syntax.........: _YTGetFormatMap($source)
; Parameters ....: $source - A YouTube video HTML source to scrape
; Requirement(s).: v3.0.102 or higher
; Return values .: Success - Returns the cleaned up fmt_url_map.
;                  Failure - Returns False.
; Author ........: Foxhound
; Example........; _YTGetFormatMap(BinaryToString(InetRead('http://www.youtube.com/watch?v=ckuGI6zH7uM')))
;=====================================================================================================================
Func _YTGetFormatMap($source)
	$fmt_url_map = _StringBetween($source,'"url_encoded_fmt_stream_map": "url=','"')
	if @error or $fmt_url_map == 0 Then return False
	return $fmt_url_map[0]
EndFunc


; #FUNCTION# =========================================================================================================
; Name...........: _YTURLDecode
; Description ...: Decodes the various html escape/special characters to form a properly formatted query string
; Syntax.........: _YTGetFormatMap($url)
; Parameters ....: $url - A valid fmt_url_map YouTube query string
; Requirement(s).: v3.0.102 or higher
; Return values .: Success - Returns the decoded URL.
;                  Failure - Returns False.
; Author ........: Foxhound
; Example........; _YTURLDecode('http:\/\/v4.lscache5.c.youtube.com/videoplayback?sparams=id%2Cexpire%2Cip%2Cipbits%2Citag%2Cratebypass%2C')
;=====================================================================================================================
Func _YTURLDecode($url)
	$url = StringReplace($url, '%3A', ':')
	$url = StringReplace($url, '%2F', '/')
	$url = StringReplace($url, '%3F', '?')
	$url = StringReplace($url, '%252C', '%2C')
	$url = StringReplace($url, '%3D', '=')
	$url = StringReplace($url, '%26', '&')
	$url = StringReplace($url, '\u0026', '&')
	If Not @error Then Return $url
	Return False
EndFunc   ;==>_YTURLDecode

; #FUNCTION# =========================================================================================================
; Name...........: _YTCheckFormat
; Description ...: Checks for valid YouTube URL using RegEx.
; Syntax.........: _YTCheckFormat($url)
; Parameters ....: $url - Required. A  URL to check if it's a valid YouTube URL: http://www.youtube.com/watch?v=xxxxxxxxxxx
; Requirement(s).: v3.0.102 or higher
; Return values .: Success - Returns True.
;                  Failure - Returns False.
; Author ........: Foxhound
; Example........; _YTCheckFormat('http://www.youtube.com/watch?v=P5VLvzl63e8') ;returns true
;=====================================================================================================================
Func _YTCheckFormat($url)
	If NOT StringRegExp($url, 'http:\/\/www\.youtube\.com\/watch\?v=[a-zA-Z0-9-_]{11}', 0) Then Return False
	Return True
EndFunc   ;==>_YTCheckFormat

; #FUNCTION# =========================================================================================================
; Name...........: _YTGetVideoIDByURL
; Description ...: Returns an 11-character YouTube ID from a URL
; Syntax.........: _YTGetVideoIDByURL($url)
; Parameters ....: $url - Required. A valid YouTube URL that contains a YouTube ID in one of the parameters
; Requirement(s).: v3.0.102 or higher
; Return values .: Success - Returns the video ID of the given URL
;                  Failure - Returns False.
; Author ........: Foxhound
; Example........; _YTGetVideoIDByURL('http://www.youtube.com/watch?v=P5VLvzl63e8') ; Returns P5VLvzl63e8
;=====================================================================================================================
Func _YTGetVideoIDByURL($url)
	Local $match
	If Not _YTCheckFormat($url) Then Return False
	$match = StringRegExp($url, 'watch\?v=[a-zA-Z0-9-_]{11}', 1)
	If IsArray($match) Then
		$match[0] = StringRight($match[0], 11)
		Return $match[0]
	EndIf
	Return False

EndFunc   ;==>_YTGetVideoIDByURL


; #FUNCTION# =========================================================================================================
; Name...........: _YTGetDownloadLink
; Description ...: Returns an 11-character YouTube ID from a URL
; Syntax.........: _YTGetDownloadLink($url)
; Parameters ....: $url - Required. A valid YouTube URL that contains a YouTube ID in one of the parameters
; Requirement(s).: v3.0.102 or higher
; Return values .: Success - Returns the video ID of the given URL
;                  Failure - Returns False.
; Author ........: Foxhound
; Example........; _YTGetDownloadLink('http://www.youtube.com/watch?v=P5VLvzl63e8') ; Returns P5VLvzl63e8
;=====================================================================================================================
func _YTGetDownloadLink($url)
	dim $fmt_map[10]

	;Get all the available download links
	$download_links = _YTGetFormatMap(BinaryToString(InetRead($url)))
	$download_links = StringSplit($download_links,",",2)

	if not IsArray($download_links) Then return False
	$link = $download_links[0]
	
	;For $element In $download_links
    ;     $string = $element & @CRLF
	;	 FileWriteLine(@ScriptDir & "\xxx.log", $string)
    ;Next
	

	; Decode the query strings
	$link =_YTURLDecode($link)

	;remove residual query strings
	$link = StringRegExpReplace($link,"quality.+","")

	;remove residual query strings
	$link = StringReplace($link,"url=","")

	return $link

EndFunc

; #FUNCTION# =========================================================================================================
; Name...........: _YTGetGDataSource
; Description ...: Returns an XML file from YouTube API with  individual video information
; Syntax.........: _YTGetGDataSource($url)
; Parameters ....: $url - Reqiured. A valid YouTube URL.
; Requirement(s).: v3.0.102 or higher
; Return values .: Success - Saves the downloaded XML in the path specified by $_YT_TEMP_XML_DIR constant and returns true.
;                  Failure - Returns False.
; Author ........: Foxhound
; Example........; _YTGetGDataSource('http://www.youtube.com/watch?v=P5VLvzl63e8')
;=====================================================================================================================
Func _YTGetGDataSource($url)
	Local $videoid = _YTGetVideoIDByURL($url)
	If Not $videoid Then Return False
	Local $ytAPIURL = 'http://gdata.youtube.com/feeds/api/videos/' & $videoid & '?v=2'
	InetGet($ytAPIURL, $_YT_TEMP_XML_DIR, 1)
	If @error Then Return False
	Return True
EndFunc   ;==>_YTGetGDataSource


; #FUNCTION# =========================================================================================================
; Name...........: _YTGetVideoInformation
; Description ...: Uses the YouTube GData API to return information about a specific YouTube video
; Syntax.........: _YTGetVideoInformation($url)
; Parameters ....: Required. $url - A valid YouTube video URL ex: http://www.youtube.com/watch?v=P5VLvzl63e8
; Requirement(s).: v3.0.102 or higher
; Return values .: Success - Returns a 0 based array containing the following video information:
;					     [0] - Video Title
;						 [1] - Author Name
;						 [2] - Description
;						 [3] - View Count
;						 [4] - Video Keywords/Tags
;						 [5] - URL Video Thumbnail Picture
;						 [6] - Likes
;						 [7] - Dislikes
;						 [8] - RTSP Protocol Streaming Video Link
;                  Failure - Returns False
; Author ........: Foxhound
; Example........; _YTGetVideoInformation('http://www.youtube.com/watch?v=P5VLvzl63e8')
;=====================================================================================================================

Func _YTGetVideoInformation($url)
	;thanks to Manadar for the info on xmlns:
	;http://www.autoitscript.com/forum/topic/127494-xmldomwrapper-youtube-gdata-madness/
	Local $vArray[9]
	Local $tmp
	Local $xmlns ="xmlns:default='http://www.w3.org/2005/Atom' " & _
					"xmlns:media='http://search.yahoo.com/mrss/' " & _
					"xmlns:gd='http://schemas.google.com/g/2005' " & _
					"xmlns:yt='http://gdata.youtube.com/schemas/2007'"
   
	_YTGetGDataSource($url)
	If Not _YTCheckFormat($url) Then Return False
	_XMLFileOpen($_YT_TEMP_XML_DIR, $xmlns)
	If @error Then Return False
	$tmp = _XMLGetValue("//default:title") ;0 - Video Title
	$tmp[1] = StringRegExpReplace($tmp[1], '[\\/:"?*<>|]', "")
	$vArray[0] = $tmp[1]
	$tmp = _XMLGetValue("//default:name") ;1 - Author Name
	$vArray[1] = $tmp[1]
	$tmp = _XMLGetValue("//media:description") ;2 - Description
	$vArray[2] = $tmp[1]
	$vArray[3] = _XMLGetAttrib("//yt:statistics", "viewCount") ;3 - View Count
	$tmp = _XMLGetValue("//media:keywords") ;4- Video Keywords/Tags
	$vArray[4] = $tmp[1]
	$vArray[5] = _XMLGetAttrib("//media:thumbnail[1]", "url") ;5 - Thumbnail url
	$vArray[6] = _XMLGetAttrib("//yt:rating", "numLikes") ;6 - Likes
	$vArray[7] = _XMLGetAttrib("//yt:rating", "numDislikes") ;7- Dislikes
	;Bugfix - if there are 0 likes/dislikes, _XMLGetAttrib returns -1. Thanks to Gerald for pointing it out.
	if $vArray[6] == -1 Then $vArray[6] = "0"
	if $vArray[7] == -1 Then $vArray[7] = "0"
	$vArray[8] = _XMLGetAttrib("//media:content[3]", "url") ;8 - rtsp protocol  Video Link
	Return $vArray
EndFunc   ;==>_YTGetVideoInformation
